untrusted comment: verify with signify key from exoticsilicon.com
RWRn5d3Yx35u02537szFQXoQJGdpEYfeCZ9s0PvdGCWKzjMogpssafa5E5cMxsexUDChwIvMfQWxFc3xPwIqGsLJDsKmBG9zOgw=
# 4096 colour patch version 1:
# Implements 4-bit direct RGB colour selection.
#
# For more details about this patch, see https://research.exoticsilicon.com/articles/console_4096

--- sys/dev/rasops/rasops.c
+++ sys/dev/rasops/rasops.c
@@ -448,6 +448,7 @@
 
 	if (ri->ri_depth == 32) {
 		ri->ri_caps |= WSSCREEN_256COL;
+		ri->ri_caps |= WSSCREEN_TRUECOLOUR;
 		}
 
 	switch (ri->ri_depth) {
--- sys/dev/rasops/rasops32.c
+++ sys/dev/rasops/rasops32.c
@@ -99,6 +99,20 @@
 	b = ri->ri_devcmap[(attr >> 16) & 0xff];
 	f = ri->ri_devcmap[(attr >> 24) & 0xff];
 
+	if (attr & WSATTR_TRUECOLOUR_FG) {
+		/*
+		 * Red channel comes from the re-purposed upper bits of the indexed attribute bits
+		 * Blue channel comes from the re-purposed lower bits of the indexed attribute bits
+		 * Green channel comes from bits 12-15 of attr
+		 */
+		f = (((attr >> 24) & 0xf0) << ri->ri_rpos);
+		f |= (((attr >> 28) & 0x0f) << ri->ri_rpos);
+		f |= (((attr >> 24) & 0x0f) << (ri->ri_bpos + 4));
+		f |= (((attr >> 24) & 0x0f) << (ri->ri_bpos));
+		f |= (((attr >> 12) & 0x0f) << (ri->ri_gpos + 4));
+		f |= (((attr >> 12) & 0x0f) << (ri->ri_gpos));
+		}
+
 	/*
 	 * Implement dim by shifting each of the red, green and blue values by
 	 * one bit.
--- sys/dev/wscons/wsdisplayvar.h
+++ sys/dev/wscons/wsdisplayvar.h
@@ -104,6 +104,7 @@
 #define WSATTR_DOUBLE_UNDERLINE	128
 #define WSATTR_INVISIBLE	256
 #define WSATTR_ITALIC		512
+#define WSATTR_TRUECOLOUR_FG	1024	/* Attribute is to be interpreted as RGB value instead of an index */
 };
 
 #define	WSSCREEN_NAME_SIZE	16
@@ -120,6 +121,7 @@
 #define WSSCREEN_BLINK		8	/* can blink */
 #define WSSCREEN_UNDERLINE	16	/* can underline */
 #define WSSCREEN_256COL		32	/* supports 256 colours */
+#define WSSCREEN_TRUECOLOUR	64	/* Supports direct 24-bit RGB values */
 };
 
 /*
--- sys/dev/wscons/wsemul_vt100_subr.c
+++ sys/dev/wscons/wsemul_vt100_subr.c
@@ -660,6 +660,7 @@
 			case 34: case 35: case 36: case 37:
 				/* fg color */
 				flags |= WSATTR_WSCOLORS;
+				flags &= ~WSATTR_TRUECOLOUR_FG;
 				fgcol = ARG(n) - 30;
 				break;
 			/*
@@ -671,6 +672,10 @@
 			 */
 #define EXIST_ARG2(i) ((edp->nargs-n)>=3)
 #define ARG2_OR_DEF(i) (EXIST_ARG2(i) ? ARG(i+2) : 0)
+#define RGB_RED (edp->nargs - n >= 3 ? ARG(n + 2) : 0)
+#define RGB_GREEN (edp->nargs - n >= 4 ? ARG(n + 3) : 0)
+#define RGB_BLUE (edp->nargs - n >= 5 ? ARG(n + 4) : 0)
+#define RGB_RANGE_VALID (RGB_RED < 256 && RGB_GREEN < 256 && RGB_BLUE < 256)
 			case 38:
 				/*
 				 * 38 followed by zero arguments is meaningless.
@@ -685,6 +690,7 @@
 				 */
 				if (ARG(n+1)==5) {
 					flags |= WSATTR_WSCOLORS;
+					flags &= ~WSATTR_TRUECOLOUR_FG;
 					if (edp->scrcapabilities &
 					    WSSCREEN_256COL) {
 						fgcol = ARG2_OR_DEF(n);
@@ -696,21 +702,26 @@
 					break;
 				}
 				/*
-				 * 2 should introduce a sequence of three
+				 * 2 introduces a sequence of up to three
 				 * arguments, specifying RGB.
 				 *
-				 * We don't, (yet!), support setting colours by
-				 * 24-bit RGB arguments and don't want to
-				 * interpret these as regular SGR codes.
+				 * Missing values are treated as zero by xterm,
+				 * so we do the same.
 				 *
-				 * If there are more then three, skip just
-				 * three, otherwise skip all of them.
+				 * This means that even CSI 38;2m is valid and
+				 * sets RGB = 0, 0, 0, I.E. black.
 				 */
 				if (ARG(n+1)==2) {
-					n=(edp->nargs-n > 5 ? n+4 :
-					    edp->nargs);
+					if (edp->scrcapabilities & WSSCREEN_TRUECOLOUR && RGB_RANGE_VALID) {
+						flags |= WSATTR_WSCOLORS;
+						flags |= WSATTR_TRUECOLOUR_FG;
+						flags &= 0x0fff; /* Clear upper bits as we are or'ing the new values in */
+						flags |= ((RGB_GREEN & 0xF0) << 8);
+						fgcol = (RGB_RED & 0xF0) | ((RGB_BLUE & 0xF0) >> 4);
+						}
+					n=(edp->nargs-n > 5 ? n+4 : edp->nargs);
 					break;
-				}
+					}
 				/*
 				 * Invalid code, I.E. not 2 or 5.
 				 *
@@ -725,6 +736,7 @@
 				fgcol = WSCOL_WHITE;
 				if (bgcol == WSCOL_BLACK)
 					flags &= ~WSATTR_WSCOLORS;
+				flags &= ~WSATTR_TRUECOLOUR_FG;
 				break;
 			case 40: case 41: case 42: case 43:
 			case 44: case 45: case 46: case 47:
@@ -765,6 +777,7 @@
 			case 94: case 95: case 96: case 97:
 				/* bright foreground color */
 				flags |= WSATTR_WSCOLORS;
+				flags &= ~WSATTR_TRUECOLOUR_FG;
 				fgcol = ARG(n) - 82;
 				break;
 			case 100: case 101: case 102: case 103:
