untrusted comment: verify with signify key from exoticsilicon.com
RWRn5d3Yx35u0yhqlyzKMze6CUg5wnhTlxzbFnuw4IjkFA+aipNU90Z8QUjKiSrqMP6I54S7P65nrimSN8j7xJGD885btUuSmA8=
# Blink attribute patch version 2:
#
# Basic implementation of the blink attribute for 32-bit rasops based displays.
# Implements blink type 0, (four-step fading)
# Implements blink type 1, (two-step dim / normal)
# Implements blink type 2, (smooth fading)
#
# For more details about this patch, see https://research.exoticsilicon.com/articles/console_4096

--- sys/dev/rasops/rasops.c
+++ sys/dev/rasops/rasops.c
@@ -30,6 +30,8 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
+extern int flag_blinking_subsystem;
+
 #include <sys/param.h>
 #include <sys/malloc.h>
 #include <sys/systm.h>
@@ -139,6 +141,8 @@
 };
 
 /* Generic functions */
+int	rasops_blink_flash_init(void *, int, int);
+void	rasops_blink_flash_do(void *);
 int	rasops_copycols(void *, int, int, int, int);
 int	rasops_copyrows(void *, int, int, int);
 int	rasops_mapchar(void *, int, u_int *);
@@ -172,6 +176,7 @@
 #endif
 
 void	rasops_doswitch(void *);
+int	rasops_vcons_blink_flash_init(void *, int, int);
 int	rasops_vcons_cursor(void *, int, int, int);
 int	rasops_vcons_mapchar(void *, int, u_int *);
 int	rasops_vcons_putchar(void *, int, int, u_int, uint32_t);
@@ -270,6 +275,7 @@
 	LIST_INIT(&ri->ri_screens);
 	ri->ri_nscreens = 0;
 
+	ri->ri_blink_flash_init = ri->ri_ops.blink_flash_init;
 	ri->ri_putchar = ri->ri_ops.putchar;
 	ri->ri_copycols = ri->ri_ops.copycols;
 	ri->ri_erasecols = ri->ri_ops.erasecols;
@@ -289,6 +295,7 @@
 		ri->ri_bs =
 		    &ri->ri_active->rs_bs[ri->ri_active->rs_dispoffset];
 
+		ri->ri_ops.blink_flash_init = rasops_vcons_blink_flash_init;
 		ri->ri_ops.cursor = rasops_vcons_cursor;
 		ri->ri_ops.mapchar = rasops_vcons_mapchar;
 		ri->ri_ops.putchar = rasops_vcons_putchar;
@@ -322,6 +329,9 @@
 	task_set(&ri->ri_switchtask, rasops_doswitch, ri);
 
 	rasops_init_devcmap(ri);
+
+	timeout_set(&ri->ri_blinkto, rasops_blink_flash_do, ri);
+
 	return (0);
 }
 
@@ -365,6 +375,8 @@
 	while ((ri->ri_emuwidth * bpp & 31) != 0)
 		ri->ri_emuwidth--;
 
+	ri->ri_blinkphase = 3;
+
 #if NRASOPS_ROTATION > 0
 	if (ri->ri_flg & (RI_ROTATE_CW | RI_ROTATE_CCW)) {
 		ri->ri_rows = ri->ri_emuwidth / ri->ri_font->fontwidth;
@@ -427,6 +439,7 @@
 	 * Fill in defaults for operations set.  XXX this nukes private
 	 * routines used by accelerated fb drivers.
 	 */
+	ri->ri_ops.blink_flash_init = rasops_blink_flash_init;
 	ri->ri_ops.mapchar = rasops_mapchar;
 	ri->ri_ops.copyrows = rasops_copyrows;
 	ri->ri_ops.copycols = rasops_copycols;
@@ -450,6 +463,10 @@
 		ri->ri_caps |= WSSCREEN_256COL;
 		}
 
+	if (ri->ri_depth == 32) {
+		ri->ri_caps |= WSSCREEN_BLINK;
+		}
+
 	switch (ri->ri_depth) {
 #if NRASOPS1 > 0
 	case 1:
@@ -543,9 +560,6 @@
 {
 	int swap;
 
-	if ((flg & WSATTR_BLINK) != 0)
-		return (EINVAL);
-
 	if ((flg & WSATTR_WSCOLORS) == 0) {
 		fg = WS_DEFAULT_FG;
 		bg = WS_DEFAULT_BG;
@@ -1822,6 +1836,7 @@
 		bg = ((u_int)attr >> 16) & 0xff;
 		attr &= ~0x0ffff0000;
 		attr |= (fg << 16) | (bg << 24);
+		attr &= ~WSATTR_BLINK;
 	}
 
 	return ri->ri_putchar(ri, ri->ri_crow, ri->ri_ccol, uc, attr);
@@ -2029,4 +2044,79 @@
 	}
 
 	return 0;
+}
+
+int rasops_blink_flash_init(void * cookie, int blinkinterval, int blinktype)
+{
+	struct rasops_info * ri;
+	ri = (struct rasops_info *) cookie;
+	/*
+	 * A value of 255 disables the blinking subsystem until reboot.
+	 */
+	if (ri->ri_blinktype == 255)
+		return 0;
+	timeout_del(&ri->ri_blinkto);
+	/*
+	 * The minimum permitted interval depends on the blinktype value:
+	 * For blinktype 2, intervals less than 30 msec are rounded to zero.
+	 * For all other blinktype values, intervals less than 500 msec are rounded to zero.
+	 */
+#define BLINK_MIN_INTERVAL (blinktype == 2 ? 30 : 500)
+	blinkinterval = ((blinkinterval < BLINK_MIN_INTERVAL || blinktype == 255) ? 0 : blinkinterval);
+	ri->ri_blinkinterval = blinkinterval;
+	ri->ri_blinktype = blinktype;
+	if (blinkinterval > 0)
+		timeout_add_msec(&ri->ri_blinkto, blinkinterval);
+	rasops_blink_flash_do(cookie);
+	return 0;
+}
+
+void rasops_blink_flash_do(void * cookie)
+{
+	struct rasops_info * ri;
+	struct rasops_screen * scr;
+	int row, col, pos;
+	ri = (struct rasops_info *) cookie;
+	if (flag_blinking_subsystem != 1 && !(flag_blinking_subsystem != 1 && ri->ri_blinkinterval == 0))
+		return ;
+	if (ri->ri_blinkinterval > 0)
+		timeout_add_msec(&ri->ri_blinkto, ri->ri_blinkinterval);
+	scr = ri->ri_active;
+	if (ri->ri_blinktype == 0)
+		ri->ri_blinkphase = (ri->ri_blinkphase + 1) % 6;
+	if (ri->ri_blinktype == 1)
+		ri->ri_blinkphase = (ri->ri_blinkphase > 0 ? 0 : 3);
+	/*
+	 * For blink type 2, we count in bits 8 - 15, but need to avoid storing
+	 * zero because this would be mis-interpreted by the code in rasops32.c.
+	 * To avoid this, we count from 1 - 255 in bits 8 - 15 instead of 0 - 255.
+	 */
+	if (ri->ri_blinktype == 2)
+		ri->ri_blinkphase = ((1 + (((ri->ri_blinkphase >> 8) + 1) % 255)) << 8);
+	/*
+	 * If we're disabling blinking, then leave blinking text as full brightness,
+	 * I.E. phase three of the blink cycle.
+	 */
+	if (ri->ri_blinkinterval == 0)
+		ri->ri_blinkphase = 3;
+	/*
+	 * Repaint screen
+	 */
+	for (row = 0; row < ri->ri_rows; row++)
+		for (col = 0; col < ri->ri_cols; col++) {
+			pos = (row * ri->ri_cols + col) + scr->rs_visibleoffset;
+			if (scr->rs_bs[pos].attr & WSATTR_BLINK)
+				/*
+				 * Don't re-paint the character under the cursor.
+				 */
+				if ((scr->rs_visibleoffset != scr->rs_dispoffset) || (scr->rs_crow != row || scr->rs_ccol != col))
+					ri->ri_putchar(ri, row, col, scr->rs_bs[pos].uc, scr->rs_bs[pos].attr);
+		}
+	return ;
+}
+
+int rasops_vcons_blink_flash_init(void * cookie, int blinkinterval, int blinktype)
+{
+	struct rasops_screen * scr = cookie;
+	return scr->rs_ri->ri_blink_flash_init(scr->rs_ri, blinkinterval, blinktype);
 }
--- sys/dev/rasops/rasops.h
+++ sys/dev/rasops/rasops.h
@@ -41,6 +41,9 @@
 
 #include "rasops_glue.h"
 
+#include <sys/types.h>
+#include <sys/timeout.h>
+
 struct wsdisplay_font;
 
 /* For rasops_info::ri_flg */
@@ -107,6 +110,10 @@
 	int	ri_xorigin;	/* where ri_bits begins (x) */
 	int	ri_yorigin;	/* where ri_bits begins (y) */
 	int32_t	ri_devcmap[256]; /* color -> framebuffer data */
+int	ri_blinkphase;	/* current stage of blinking cycle */
+int	ri_blinkinterval;	/* blinking cycle step time in msec */
+int	ri_blinktype;	/* blink type - on and off, or ramping */
+struct timeout ri_blinkto;
 
 	/* The emulops you need to use, and the screen caps for wscons */
 	struct	wsdisplay_emulops ri_ops;
@@ -136,6 +143,7 @@
 	int	(*ri_copyrows)(void *, int, int, int);
 	int	(*ri_eraserows)(void *, int, int, uint32_t);
 	int	(*ri_pack_attr)(void *, int, int, int, uint32_t *);
+	int	(*ri_blink_flash_init)(void *, int, int);
 };
 
 #define DELTA(p, d, cast) ((p) = (cast)((caddr_t)(p) + (d)))
--- sys/dev/rasops/rasops32.c
+++ sys/dev/rasops/rasops32.c
@@ -99,6 +99,33 @@
 	b = ri->ri_devcmap[(attr >> 16) & 0xff];
 	f = ri->ri_devcmap[(attr >> 24) & 0xff];
 
+	if ((attr & WSATTR_BLINK) != 0) {
+		if (ri->ri_blinkphase==0) {
+			f=(f>>3) & 0x1F1F1F1F;
+			b=(b>>3) & 0x1F1F1F1F;
+		}
+		if (ri->ri_blinkphase==1 || ri->ri_blinkphase==5) {
+			f=(f>>2) & 0x3F3F3F3F;
+			b=(b>>2) & 0x3F3F3F3F;
+		}
+		if (ri->ri_blinkphase==2 || ri->ri_blinkphase==4) {
+			f=(f>>1) & 0x7F7F7F7F;
+			b=(b>>1) & 0x7F7F7F7F;
+		}
+#define BP_FADE ((ri->ri_blinkphase >> 8) < 128 ? (ri->ri_blinkphase >> 8) : 255 - (ri->ri_blinkphase >> 8))
+#define BP_FADE_CLAMP(i) ((i - BP_FADE) > 0 ? ((i - BP_FADE)) : 0)
+		if (ri->ri_blinkphase > 255) {
+			j = (((BP_FADE_CLAMP(((f & 0x00FF0000) >> 16))) << 16) & 0x00FF0000);
+			j |= (((BP_FADE_CLAMP(((f & 0x0000FF00) >> 8))) << 8) & 0x0000FF00);
+			j |= (((BP_FADE_CLAMP(((f & 0x000000FF) >> 0))) << 0) & 0x000000FF);
+			f = j;
+			j = (((BP_FADE_CLAMP(((b & 0x00FF0000) >> 16))) << 16) & 0x00FF0000);
+			j |= (((BP_FADE_CLAMP(((b & 0x0000FF00) >> 8))) << 8) & 0x0000FF00);
+			j |= (((BP_FADE_CLAMP(((b & 0x000000FF) >> 0))) << 0) & 0x000000FF);
+			b = j;
+		}
+	}
+
 	/*
 	 * Implement dim by shifting each of the red, green and blue values by
 	 * one bit.
--- sys/dev/wscons/wsdisplayvar.h
+++ sys/dev/wscons/wsdisplayvar.h
@@ -84,6 +84,7 @@
 		    uint32_t *attrp);
 	void	(*unpack_attr)(void *c, uint32_t attr, int *fg, int *bg,
 		    int *ul);
+	int	(*blink_flash_init)(void *c, int interval, int blinktype);
 /* fg / bg values. Made identical to ANSI terminal color codes. */
 #define WSCOL_BLACK	0
 #define WSCOL_RED	1
--- sys/dev/wscons/wsemul_vt100_subr.c
+++ sys/dev/wscons/wsemul_vt100_subr.c
@@ -615,6 +615,13 @@
 			case 5: /* blink */
 				flags |= WSATTR_BLINK;
 				break;
+case 128: /* initialise and/or configure blinking */
+	WSEMULOP(m, edp, &edp->abortstate, blink_flash_init, (edp->emulcookie, ((edp->nargs > n+1) ? ARG(n + 1) : 0), ((edp->nargs > n+2) ? ARG(n + 2) : 0)));
+	if (edp->nargs > n + 1)
+		n++;
+	if (edp->nargs > n + 1)
+		n++;
+	break;
 			case 7: /* reverse */
 				flags |= WSATTR_REVERSE;
 				break;
--- sys/kern/kern_sysctl.c
+++ sys/kern/kern_sysctl.c
@@ -35,6 +35,8 @@
  *	@(#)kern_sysctl.c	8.4 (Berkeley) 4/14/94
  */
 
+int flag_blinking_subsystem = 0;
+
 /*
  * sysctl system call.
  */
@@ -622,6 +624,8 @@
 	case KERN_AUTOCONF_SERIAL:
 		return (sysctl_bounded_arr(kern_vars, nitems(kern_vars), name,
 		    namelen, oldp, oldlenp, newp, newlen));
+	case KERN_BLINKING_SUBSYSTEM:
+		return (sysctl_int(oldp, oldlenp, newp, newlen, &flag_blinking_subsystem));
 	}
 
 	savelen = *oldlenp;
--- sys/sys/sysctl.h
+++ sys/sys/sysctl.h
@@ -193,7 +193,8 @@
 #define	KERN_VIDEO		89	/* struct: video properties */
 #define	KERN_CLOCKINTR		90	/* node: clockintr */
 #define	KERN_AUTOCONF_SERIAL	91	/* int: kernel device tree state serial */
-#define	KERN_MAXID		92	/* number of valid kern ids */
+#define	KERN_BLINKING_SUBSYSTEM	92	/* flag to enable or disable the blinking subsystem */
+#define	KERN_MAXID		93	/* number of valid kern ids */
 
 #define	CTL_KERN_NAMES { \
 	{ 0, 0 }, \
@@ -288,6 +289,7 @@
 	{ "video", CTLTYPE_STRUCT }, \
  	{ "clockintr", CTLTYPE_NODE }, \
 	{ "autoconf_serial", CTLTYPE_INT }, \
+	{ "blinking_subsystem", CTLTYPE_INT }, \
 }
 
 /*
